function model = FitModel(neuronresponse,varargin);
% neuronresponse is a structure with fields background (1x1), monocL, monocR (both 1x6), binoc (6x6)
% This version fits General RFs

if nargin==1
    initguess = [ ones(1,12) ];
else
    initguess = varargin{1};
end
if length(initguess)==12
    fprintf('Fitting a model with 12 free parameters, representing monocular responses only. Tonic input is derived from background and output exponent is assumed to be 1.')
elseif length(initguess)==13
    fprintf('Fitting a model with 13 free parameters, representing monocular responses and output exponent. Tonic input is derived from background rather than allowed to be free.')
elseif length(initguess)==14
    fprintf('Fitting a model with 14 free parameters, representing monocular responses, output exponent and tonic input.')
end

% Convert it into an array:
dataarray = MakeArray(neuronresponse);


options1 = optimset('display','off');
options2 = optimset('maxfunevals',1e6,'display','off');
[err,err_noreg] = tominimise(initguess);
fprintf('Error of the initial guess provided:  with regularization = %4.3f and without it = %4.3f\n',err,err_noreg)
[params,fval,exitflag] = fminsearch(@tominimise,initguess,options1);
[params,fval,exitflag] = fminsearch(@tominimise,params,options2);
[err,err_noreg] = tominimise(params);
fprintf('Error fitting (this should be lower!): with regularization = %4.3f and without it = %4.3f\n',err,err_noreg)

% Also try starting from moncoular responses
L = neuronresponse.monocL - neuronresponse.background;
R = neuronresponse.monocR - neuronresponse.background;
params2 = params;
params2(1:12) = [L R];
[err,err_noreg] = tominimise(params2);
fprintf('Error of initial guess based on monoc responses: with regularization = %4.3f and without it = %4.3f\n',err,err_noreg)
[params2,fval2,exitflag] = fminsearch(@tominimise,params2,options1);
[params2,fval2,exitflag] = fminsearch(@tominimise,params2,options2);
[err,err_noreg] = tominimise(params2);
fprintf('Error after fitting (this should be lower!):    with regularization = %4.3f and without it = %4.3f\n',err,err_noreg)

if fval2<fval
    params=params2;
    fval=fval2;
end
model = GetModelFromParams(params);
model.fiterror = fval;
[err,err_noreg] = tominimise(params);
model.fiterrorchk = err;
model.fiterror_noreg = err_noreg;
model.response = GetModelOutput(model)  ;

% Get % variance explained
modelarray  = MakeArray(model.response);
TV = var(dataarray);
RV = var(dataarray-modelarray);
PV = 100 * (TV-RV) / TV;
model.percentVarianceExplained = PV;


    function array = MakeArray(response)
        % Combine the fields into one long array:
        array  = [response.background response.monocL response.monocR response.binoc(:)'];
    end

    function [fiterror,fiterror_noreg] = tominimise(params)
        % Build a model from the params argument
        trymodel = GetModelFromParams(params);
        
        % Get the response of this model to our stimuli:
        modelresponse = GetModelOutput(trymodel);
        % 
        modelarray  = MakeArray(modelresponse);
        % Compare with experimental data: Use logs to fit frractional error
        modeldatadiff = ((modelarray) - (dataarray));       
        fiterror_noreg = sum(modeldatadiff.^2);
        fiterror = fiterror_noreg + 0.001*sum(params.^2);% + (10*params(11)).^2; % sum-squared-error plus regularisation term
    end
    function model = GetModelFromParams(params);
        % BUild a model structure from an array of parameters (used in fitting)
        
        model.RFL = params(1:6); % response of the RF to the six different bar positions
        model.RFR = params(7:12); % response of the RF to the six different bar positions
        if length(params)>12
            model.outputexponent = abs(params(13));
        else
            model.outputexponent = 1;
        end
        model.threshold = 0;
        if length(params)>13
            model.tonicinput = params(14);%
        else
            model.tonicinput = model.threshold + neuronresponse.background^(1/model.outputexponent);
        end
        
        % response = (Pos( L + R + tonicinput - threshold) )^outputexponent
        % background = (Pos( tonicinput - threshold) )^outputexponent
        % background^(1/outputexponent) =  tonicinput - threshold
        % background^(1/outputexponent) + threshold =  tonicinput 
        
    end

end
